// ArrayObject.cpp : Implementation of CArrayObject
#include "stdafx.h"
#include "SADemo.h"
#include "ArrayObject.h"
#include "CComVector.h" // Chris Sells' SAFEARRAY bits

/////////////////////////////////////////////////////////////////////////////
// CArrayObject


STDMETHODIMP CArrayObject::getListFromEnterprise(BSTR bstrEnterpriseName, VARIANT *pStoreList)
{
	// Check the pointer
	if ( pStoreList == NULL ) return E_POINTER;

	// Original code would search Active Directory
	// using value identified in "bstrEnterpriseName"
	//
	// A list of stores is returned through the
	// pStoreList argument, which is a VARIANT 
	// of type VT_SAFEARRAY, filled with VARIANTS
	// of type VT_BSTR.  Here is how you create and
	// manage the SAFEARRAY...
	HRESULT hr = S_OK;
	try {
		long lNumParms = 2; // set as appropriate...
		CComVector<BSTR> v(lNumParms);
		if ( !v ) throw E_OUTOFMEMORY;

		CComVectorData<BSTR> rg(v);
		if ( !rg ) throw E_OUTOFMEMORY;

		// Here you add the strings to the array...
		// We just needed a hardcode test, but you
		// could use any technique you like...
		rg[0] = SysAllocString(L"foo");
		rg[1] = SysAllocString(L"bar");

		// Stuff SAFEARRAY into a VARIANT
		CComVariant vt;
		vt.vt = VT_SAFEARRAY;
		vt.parray = NULL; // DetachTo() will try to destroy non-NULL ptr!
		v.DetachTo(&vt.parray);

		// Return local copy.  Note this uses memcpy()
		// rather than ::VariantCopy()...
		hr = vt.Detach(pStoreList);
		if ( FAILED(hr) ) throw hr;
	} // try
	catch(HRESULT hrErr) {
		// Some COM error
		hr = hrErr;
		ATLTRACE("SADemo: COM error %#08x in getListFromEnterprise()\n",hrErr);
	} // catch
	catch(...) {
		// Some other error
		hr = E_UNEXPECTED;
		ATLTRACE("SADemo: unexpected error in getListFromEnterprise()\n");
	} // catch

	return hr;
}
